@@ -18,25 +18,18 @@ class AwsRegistry:
         self.client_docker = client_docker or docker.from_env()
 
     def get_all_image_tags(self, image_name, tag_filter='', with_deleted=False):
-        images = self.client_aws.list_images(repositoryName=image_name)
+        images = self.client_aws.describe_images(repositoryName=image_name)
+
         results = []
-        for image in images['imageIds']:
-            if 'imageTag' not in image:
+        for image in images['imageDetails']:
+            if not 'imageTags' in image:
                 continue
-            tag = image['imageTag']
-            content_match = re.search(tag_filter, tag) if not with_deleted else \
-                re.search(tag_filter, tag) or re.search(self.DELETE_PREFIX_TAG + tag_filter, tag)
-            if content_match:
-                img = self.client_aws.describe_images(
-                    repositoryName=image_name,
-                    imageIds=[
-                        {
-                            'imageTag': tag
-                        },
-                    ]
-                )
-                pushed_at = img['imageDetails'][0]['imagePushedAt']
-                results.append({'tag': tag, 'pushed_at': pushed_at, 'image': image})
+            for tag in image['imageTags']:
+                content_match = re.search(tag_filter, tag) if not with_deleted else \
+                    re.search(tag_filter, tag) or re.search(self.DELETE_PREFIX_TAG + tag_filter, tag)
+                if content_match:
+                    pushed_at = image['imagePushedAt']
+                    results.append({'tag': tag, 'pushed_at': pushed_at, 'image': image})
         results = sorted(results, key=lambda x: x['pushed_at'], reverse=True)
         return results
 
@@ -46,7 +39,7 @@ class AwsRegistry:
         for image in images:
             digest = image['image']['imageDigest']
             if digest.startswith(hash_filter):
-                hash_dict[digest].append(image['tag'])
+                hash_dict[digest] = image
         return hash_dict
 
     def pull_image(self, image_name, tag=None):
