@@ -1,10 +1,11 @@
+import copy
 from semantic_version import Version
 
 from cx_releaser.src.docker_registry import AwsRegistry
 
 
 class Release:
-    version_pattern = '[0-9]\.[0-9]\.[0-9]'
+    version_pattern = '[0-9]+\.[0-9]+\.[0-9]+'
 
     def __init__(self, registry_client: AwsRegistry, name, version,
                  local_name=None, equal_tags=None,
@@ -24,6 +25,7 @@ class Release:
         self.local_name = local_name or f'{self.name}:{self.version}'
         self.incr_version = incr_version
         self.hash = None
+        self.pushed_at = None
         self._next = None
         self._prev = None
 
@@ -31,20 +33,21 @@ class Release:
         equal_tags = []
         hashes = self.registry_client.get_images_on_unique_hash(self.name, hash_filter=hash_filter, images=images)
         new_hash = None
-        for hash, tags in hashes.items():
+        for hash, img in hashes.items():
+            tags = img['image']['imageTags']
             if self.version in tags:
                 new_hash = hash
                 equal_tags.extend(tags)
         if equal_tags:
             equal_tags.remove(self.version)
         if with_hash:
-            return equal_tags, new_hash
+            return equal_tags, hashes.get(new_hash)
         return equal_tags
 
     def exist_on_remote(self):
         hashes = self.registry_client.get_images_on_unique_hash(self.name, hash_filter='')
         for hash, tags in hashes.items():
-            if self.version in tags:
+            if self.version in tags['image']['imageTags']:
                 return True
         return False
 
@@ -80,9 +83,9 @@ class Release:
         if test_tag:
             return self.registry_client.push_image(self.name, test_tag, local_name=self.local_name)
         validate and self.validate_push(check_is_next=
-                                        'validate_is_next' in checks,
+                                        'check_is_newest' in checks,
                                         check_new_hash=
-                                        'validate_new+hash' in checks)
+                                        'check_is_new_hash' in checks)
         result = self.registry_client.push_image(self.name, self.version, local_name=self.local_name)
         for tag in self.equal_tags:
             self.registry_client.push_image(self.name, str(tag), local_name=self.local_name)
@@ -103,9 +106,7 @@ class Release:
                 to_move.extend([tag for tag in tags_to_move if tag not in prev_release.equal_tags])
                 to_move = list(set(to_move))
                 prev_release.update_tags(to_move)
-        if tags_to_move:
-            to_move.extend(tags_to_move)
-        self.delete(with_equal_tags=True, excl_tags=to_move)
+        return self.delete(with_equal_tags=True, excl_tags=to_move)
 
     def prev(self, unique_hash=True, prev_id=1):
         res = self.get_all_from_remote(self.name, self.registry_client,
@@ -118,8 +119,10 @@ class Release:
 
     def sync_with_remote(self, images=None, version=False):
         if version:
-            self.version = self.get_versions(last=True)
-        self.equal_tags, self.hash = self.get_equal_tags_from_remote(with_hash=True, images=images)
+            self.version = str(self.get_versions(last=True))
+        self.equal_tags, data = self.get_equal_tags_from_remote(with_hash=True, images=images)
+        digset, date = data['image']['imageDigest'], data['pushed_at']
+        self.hash, self.pushed_at = digset, date
         return self
 
     def delete(self, with_equal_tags=True, incl_tags=None, excl_tags=None):
@@ -158,10 +161,11 @@ class Release:
                                                                 hash_filter=digset)
         self.delete_tags([test_tag])
         if result:
-            for dig, tags in result.items():
+            for dig, img in result.items():
+                tags = img['image']['imageTags']
                 if tags != [test_tag]:
                     raise ValueError(
-                        f'Already {len(list(result.values())[0])} tags: {list(result.values())[0]} with the same hash: {digset} has been deployed')
+                        f'Already {len(list(result.values())[0])} tags: {tags} with the same hash: {digset} has been deployed')
         return True
 
     def get_versions(self, last=True):
@@ -174,40 +178,50 @@ class Release:
             return versions[0] if versions else Version.coerce('0.0.0')
         return versions
 
-    def next(self, incr_version=None, remote_sync=False):
+    def next(self, incr_version=None, remote_sync=False, inplace=False):
         if remote_sync:
             version = self.get_versions(last=True)
         else:
             version = self.version_sem()
-        position = incr_version or self.incr_version
-        self.version = str(getattr(version, f'next_{position}')())
-        return self
+        if inplace:
+            obj = self
+        else:
+            obj = copy.copy(self)
+        position = incr_version or obj.incr_version
+        obj.version = str(getattr(version, f'next_{position}')())
+        return obj
 
     @classmethod
     def from_remote(cls, name, client: AwsRegistry,
-                    local_name=None):
-        images = client.get_all_image_tags(name, tag_filter=cls.version_pattern)
-        if not images:
-            raise ValueError(f'Could not fetch from remote. No versions found for pattern {cls.version_pattern}')
-        newest = images[0]['tag']
-        obj = cls(client, name, newest, local_name=local_name)
-        obj.sync_with_remote()
-        return obj
+                    local_name=None, sort_key='tag'):
+        results = cls.get_all_from_remote(name, client, local_name,
+                                          unique_hashes=False,
+                                          sort_key=sort_key)
+        if not results:
+            raise ValueError(f"Remote release does not exist for {name}")
+        return results[0]
 
     @classmethod
     def get_all_from_remote(cls, name,
                             client: AwsRegistry, local_name=None,
-                            unique_hashes=False):
+                            unique_hashes=False, sort_key='tag'):
         images_version = client.get_all_image_tags(name, tag_filter=cls.version_pattern)
         images = client.get_all_image_tags(name, tag_filter='')
         results = []
+        hashes = set()
         for image in images_version:
             obj = cls(client, name, image['tag'], local_name=local_name)
             obj.sync_with_remote(images)
+            if unique_hashes and obj.hash in hashes:
+                continue
             results.append(obj)
-        if unique_hashes:
-            results = list(set(results))
-        return sorted(results, reverse=True)
+        if sort_key == 'tag':
+            return sorted(results, reverse=True)
+        elif sort_key == 'pushed_at':
+            return results
+        else:
+            ValueError('Sort key can be tag or pushed_at')
+        return results
 
     @classmethod
     def from_local(cls, name, client: AwsRegistry,
